#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## Bookkun - Manage your ebooks.                                          ##
##                                                                        ##
## Copyright (C) 2012  Fabien Loison <flo AT flogisoft DOT com>           ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################


from bookkun.formats.epub import Epub


FORMATS = {
        'epub': Epub,
        }


def open_ebook(path):
    """Open the given eBook.

    This function try to open the given eBook with available Format.

    Return:
        An instance of Format object with the eBook opened or None.
    """
    ext = path.split('.')[-1].lower()
    if ext in FORMATS:
        ebook = FORMATS[ext](path)
        if ebook.can_handle():
            try:
                ebook.open()
                return ebook
            except Exception:
                pass
    for format_ in FORMATS.values():
        ebook = format_(path)
        if ebook.can_handle():
            try:
                ebook.open()
                return ebook
            except Exception:
                pass
    return None


class Format(object):

    """Handle a specific eBook format (base class).

    Args:
        path -- The path of the eBook file.
    """

    _path = None

    def __init__(self, path):
        """The constructor."""
        self._path = path

    def can_handle(self):
        """Check if the eBook can be handled by this Format.

        Return:
            True if the eBook can be handled, False else.
        """
        raise NotImplementedError()

    def open(self):
        """Open the eBook file."""
        raise NotImplementedError()

    def get_title(self):
        """Get the title of the eBook."""
        raise NotImplementedError()

    def get_searchable_title(self):
        """Get the searchable string of the eBook's title."""
        raise NotImplementedError()

    def get_author(self):
        """Get the author of the eBook."""
        raise NotImplementedError()

    def get_searchable_author():
        """Get the searchable string of the eBook's author."""
        raise NotImplementedError()

    def get_editor():
        """Get the editor of the eBook."""
        raise NotImplementedError()

    def get_isbn():
        """Get the ISBN number of the eBook."""
        raise NotImplementedError()

    def get_path():
        """Get the path of the eBook file."""
        return self._path

    def get_size():
        """Get the eBooks's file size."""
        pass  # TODO

    def get_thumbnail():
        """Get the eBook's thumbnail path."""
        pass  # TODO

    def _gen_thumbnail(self, path):
        """Generates the thumbnail of the eBook.

        Args:
            path -- The output file path.

        Return:
            True if the thumbnail was generated, False else.
        """
        raise NotImplementedError()
