#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## Bookkun - Manage your ebooks.                                          ##
##                                                                        ##
## Copyright (C) 2012  Fabien Loison <flo AT flogisoft DOT com>           ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################


import os

import epub

from bookkun.formats import Format
from bookkun.helpers import searchable_string


class Epub(Format):

    """Handle ePub files.

    Args:
        path -- The path of the eBook file.
    """

    _epub = None

    def can_handle(self):
        """Check if the eBook can be handled by this Format.

        Return:
            True if the eBook can be handled, False else.
        """
        try:
            self.open()
            return True
        except Exception:
            return False

    def open(self):
        """Open the eBook file."""
        if self._epub == None:
            self._epub = epub.open_epub(self._path)

    def get_title(self):
        """Get the title of the eBook."""
        titles = self._epub.opf.metadata.titles
        if len(titles) > 0:
            return titles[0][0]  # FIXME
        return self._path.split(os.sep)[-1]

    def get_searchable_title(self):
        """Get the searchable string of the eBook's title."""
        return searchable_string(self.get_title())

    def get_author(self):
        """Get the author of the eBook."""
        creators = self._epub.opf.metadata.creators
        for name, role, fileas in creators:
            if role == "aut":
                return self._format_author(name)
        if len(creators) > 0:
            return self._format_author(creators[0][0])
        return None

    def get_searchable_author():
        """Get the searchable string of the eBook's author."""
        creators = self._epub.opf.metadata.creators
        author = None
        for name, role, fileas in creators:
            if role == "aut":
                author = "%s%s" % (name, fileas)
        if len(creators) > 0:
            author = "%s%s" % (creators[0][0], creators[0][2])
        if author:
            return searchable_string(author)
        return None

    def get_editor():
        """Get the editor of the eBook."""
        raise NotImplementedError()  # TODO

    def get_isbn():
        """Get the ISBN number of the eBook."""
        raise NotImplementedError()  # TODO

    def _gen_thumbnail(self, path):
        """Generates the thumbnail of the eBook.

        Args:
            path -- The output file path.

        Return:
            True if the thumbnail was generated, False else.
        """
        raise NotImplementedError()  # TODO

    def _format_author(self, name):
        """Format the author's name.

        Return:
            The formated name
        """
        if "," in name:
            try:
                lastname, firstname = name.split(",")
                return "%s %s" % (firstname.strip().title(), lastname.strip().upper())
            except Exception:
                pass
        return name.title()
